@@ -26,7 +26,6 @@
 #include "clock-location.h"
 #include "clock-marshallers.h"
 #include "set-timezone.h"
-#include "gweather-xml.h"
 
 G_DEFINE_TYPE (ClockLocation, clock_location, G_TYPE_OBJECT)
 
@@ -51,6 +50,7 @@ typedef struct {
 
 enum {
 	WEATHER_UPDATED,
+	SET_CURRENT,
 	LAST_SIGNAL
 };
 
@@ -232,6 +232,7 @@ guess_zone_from_tree (const gchar *localtime, ClockZoneTable *zones)
 }
 
 static gchar *current_zone = NULL;
+static ClockLocation *current_location = NULL;
 static GFileMonitor *monitor = NULL;
 
 static void
@@ -409,6 +410,15 @@ clock_location_class_init (ClockLocationClass *this_class)
 			      _clock_marshal_VOID__POINTER,
 			      G_TYPE_NONE, 1, G_TYPE_POINTER);
 
+	location_signals[SET_CURRENT] = 
+		g_signal_new ("set-current",
+			      G_OBJECT_CLASS_TYPE (g_obj_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (ClockLocationClass, set_current),
+			      NULL, NULL,
+			      _clock_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
+
         g_type_class_add_private (this_class, sizeof (ClockLocationPrivate));
 }
 
@@ -482,7 +492,7 @@ clock_location_finalize (GObject *g_obj)
         G_OBJECT_CLASS (clock_location_parent_class)->finalize (g_obj);
 }
 
-gchar *
+const gchar *
 clock_location_get_name (ClockLocation *loc)
 {
         ClockLocationPrivate *priv = PRIVATE (loc);
@@ -629,15 +639,39 @@ clock_location_localtime (ClockLocation *loc, struct tm *tm)
 }
 
 gboolean
-clock_location_is_current (ClockLocation *loc)
+clock_location_is_current_timezone (ClockLocation *loc)
 {
         ClockLocationPrivate *priv = PRIVATE (loc);
 	const char *zone;
 
 	if ((zone = zone_from_etc_sysconfig_clock ()))
 		return strcmp (zone, priv->timezone) == 0;
+	else
+		return clock_location_get_offset (loc) == 0;
+}
 
-	return clock_location_get_offset (loc) == 0;
+gboolean
+clock_location_is_current (ClockLocation *loc)
+{
+	if (current_location == loc)
+		return TRUE;
+	else if (current_location != NULL)
+		return FALSE;
+
+	if (clock_location_is_current_timezone (loc)) {
+		/* Note that some code in clock.c depends on the fact that
+		 * calling this function can set the current location if
+		 * there's none */
+		current_location = loc;
+		g_object_add_weak_pointer (G_OBJECT (current_location), 
+					   (gpointer *)&current_location);
+		g_signal_emit (current_location, location_signals[SET_CURRENT],
+			       0, NULL);
+
+		return TRUE;
+	}
+
+	return FALSE;
 }
 
 
@@ -700,6 +734,15 @@ make_current_cb (gpointer data, GError *error)
  	 	 */
 		g_free (current_zone);
 		current_zone = g_strdup (priv->timezone);
+
+		if (current_location)
+			g_object_remove_weak_pointer (G_OBJECT (current_location), 
+						      (gpointer *)&current_location);
+		current_location = mcdata->location;
+		g_object_add_weak_pointer (G_OBJECT (current_location), 
+					   (gpointer *)&current_location);
+		g_signal_emit (current_location, location_signals[SET_CURRENT],
+			       0, NULL);
 	}
 
 	if (mcdata->callback)
@@ -730,6 +773,22 @@ clock_location_make_current (ClockLocation *loc,
         gchar *filename;
 	MakeCurrentData *mcdata;
 
+	if (clock_location_is_current_timezone (loc)) {
+		if (current_location)
+			g_object_remove_weak_pointer (G_OBJECT (current_location), 
+						      (gpointer *)&current_location);
+		current_location = loc;
+		g_object_add_weak_pointer (G_OBJECT (current_location), 
+					   (gpointer *)&current_location);
+		g_signal_emit (current_location, location_signals[SET_CURRENT],
+			       0, NULL);
+		if (callback)
+               		callback (data, NULL);
+		if (destroy)
+			destroy (data);	
+		return;
+	}
+
 	mcdata = g_new (MakeCurrentData, 1);
 
 	mcdata->location = g_object_ref (loc);
@@ -983,7 +1042,7 @@ setup_weather_updates (ClockLocation *loc)
 
 	dms = rad2dms (priv->latitude, priv->longitude);
 	wl = weather_location_new (priv->name, priv->weather_code,
-				   NULL, NULL, dms);
+				   NULL, NULL, dms, NULL, NULL);
 
 	priv->weather_info =
 		weather_info_new (wl, &prefs, weather_info_updated, loc);
