@@ -31,13 +31,13 @@ G_GNUC_INTERNAL void _clock_marshal_VOID__OBJECT_STRING (GClosure     *closure,
                                                          gpointer      invocation_hint,
                                                          gpointer      marshal_data);
 
-/* STRING:VOID (clock-marshallers.list:6) */
-G_GNUC_INTERNAL void _clock_marshal_STRING__VOID (GClosure     *closure,
-                                                  GValue       *return_value,
-                                                  guint         n_param_values,
-                                                  const GValue *param_values,
-                                                  gpointer      invocation_hint,
-                                                  gpointer      marshal_data);
+/* INT:VOID (clock-marshallers.list:6) */
+G_GNUC_INTERNAL void _clock_marshal_INT__VOID (GClosure     *closure,
+                                               GValue       *return_value,
+                                               guint         n_param_values,
+                                               const GValue *param_values,
+                                               gpointer      invocation_hint,
+                                               gpointer      marshal_data);
 
 G_END_DECLS
 
